@@ -1,7 +1,8 @@
 /*
  *  Off-the-Record Messaging plugin for pidgin
- *  Copyright (C) 2004-2008  Ian Goldberg, Rob Smits,
- *                           Chris Alexander, Nikita Borisov
+ *  Copyright (C) 2004-2012  Ian Goldberg, Rob Smits,
+ *                           Chris Alexander, Willy Lew,
+ *                           Nikita Borisov
  *                           <otr@cypherpunks.ca>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef __OTRG_DIALOGS_H__
@@ -34,6 +35,8 @@
 #define SESSIONID_HELPURL     BASE_HELPURL "sessionid.php"
 #define UNVERIFIED_HELPURL    BASE_HELPURL "unverified.php"
 #define LEVELS_HELPURL        BASE_HELPURL "levels.php"
+#define SESSIONS_HELPURL      BASE_HELPURL "sessions.php"
+
 
 typedef struct s_OtrgDialogWait *OtrgDialogWaitHandle;
 
@@ -47,7 +50,7 @@ typedef struct {
 	const char *title, const char *primary, const char *secondary);
 
     int (*display_otr_message)(const char *accountname, const char *protocol,
-	    const char *username, const char *msg);
+	    const char *username, const char *msg, int force_create);
 
     OtrgDialogWaitHandle (*private_key_wait_start)(const char *account,
 	const char *protocol);
@@ -62,7 +65,8 @@ typedef struct {
     void (*socialist_millionaires)(ConnContext *context, char *question,
 	    gboolean responder);
 
-    void (*update_smp)(ConnContext *context, double progress_level);
+    void (*update_smp)(ConnContext *context, OtrlSMPEvent smp_event,
+	    double progress_level);
 
     void (*connected)(ConnContext *context);
 
@@ -117,8 +121,9 @@ void otrg_dialog_notify_info(const char *accountname, const char *protocol,
 /* Display an OTR control message for the given accountname / protocol /
  * username conversation.  Return 0 on success, non-0 on error (in which
  * case the message will be displayed inline as a received message). */
-int otrg_dialog_display_otr_message( const char *accountname,
-	const char *protocol, const char *username, const char *msg);
+int otrg_dialog_display_otr_message(const char *accountname,
+	const char *protocol, const char *username, const char *msg,
+	int force_create);
 
 /* Put up a Please Wait dialog. This dialog can not be cancelled.
  * Return a handle that must eventually be passed to
@@ -146,7 +151,8 @@ void otrg_dialog_socialist_millionaires_q(ConnContext *context,
 	char *question);
 
 /* Update the status of an ongoing socialist millionaires protocol. */
-void otrg_dialog_update_smp(ConnContext *context, double progress_level);
+void otrg_dialog_update_smp(ConnContext *context, OtrlSMPEvent smp_event,
+	double progress_level);
 
 /* Call this when a context transitions to ENCRYPTED. */
 void otrg_dialog_connected(ConnContext *context);
