@@ -1,7 +1,8 @@
 /*
  *  Off-the-Record Messaging plugin for pidgin
- *  Copyright (C) 2004-2008  Ian Goldberg, Rob Smits,
- *                           Chris Alexander, Nikita Borisov
+ *  Copyright (C) 2004-2012  Ian Goldberg, Rob Smits,
+ *                           Chris Alexander, Willy Lew,
+ *                           Nikita Borisov
  *                           <otr@cypherpunks.ca>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /* config.h */
@@ -88,41 +89,46 @@ static void account_menu_changed_cb(GtkWidget *item, PurpleAccount *account,
     const char *accountname;
     const char *protocol;
     GtkWidget *fprint = ui_layout.fprint_label;
-    char s[100];
+    char *s = NULL;
     char *fingerprint;
-    
+
     if (account) {
-	char fingerprint_buf[45];
+	char fingerprint_buf[OTRL_PRIVKEY_FPRINT_HUMAN_LEN];
 	accountname = purple_account_get_username(account);
 	protocol = purple_account_get_protocol_id(account);
 	fingerprint = otrl_privkey_fingerprint(otrg_plugin_userstate,
 		fingerprint_buf, accountname, protocol);
 
 	if (fingerprint) {
-	    sprintf(s, _("Fingerprint: %.80s"), fingerprint);
+	    s = g_strdup_printf(_("Fingerprint: %.80s"), fingerprint);
 	    if (ui_layout.generate_button)
 		gtk_widget_set_sensitive(ui_layout.generate_button, 0);
 	} else {
-	    sprintf(s, _("No key present"));
+	    s = g_strdup(_("No key present"));
 	    if (ui_layout.generate_button)
 		gtk_widget_set_sensitive(ui_layout.generate_button, 1);
 	}
     } else {
-	sprintf(s, _("No account available"));
+	s = g_strdup(_("No account available"));
 	if (ui_layout.generate_button)
 	    gtk_widget_set_sensitive(ui_layout.generate_button, 0);
     }
     if (fprint) {
-	gtk_label_set_text(GTK_LABEL(fprint), s);
+	gtk_label_set_text(GTK_LABEL(fprint), s ? s : "");
 	gtk_widget_show(fprint);
     }
+    if (s) {
+	g_free(s);
+    }
 }
 
 /* Call this function when the DSA key is updated; it will redraw the
  * UI, if visible. */
 static void otrg_gtk_ui_update_fingerprint(void)
 {
-    g_signal_emit_by_name(G_OBJECT(ui_layout.accountmenu), "changed");
+    if (ui_layout.accountmenu) {
+	g_signal_emit_by_name(G_OBJECT(ui_layout.accountmenu), "changed");
+    }
 }
 
 static void account_menu_added_removed_cb(PurpleAccount *account, void *data)
@@ -132,10 +138,18 @@ static void account_menu_added_removed_cb(PurpleAccount *account, void *data)
 
 static void clist_all_unselected(void)
 {
-    gtk_widget_set_sensitive(ui_layout.connect_button, 0);
-    gtk_widget_set_sensitive(ui_layout.disconnect_button, 0);
-    gtk_widget_set_sensitive(ui_layout.forget_button, 0);
-    gtk_widget_set_sensitive(ui_layout.verify_button, 0);
+    if (ui_layout.connect_button) {
+	gtk_widget_set_sensitive(ui_layout.connect_button, 0);
+    }
+    if (ui_layout.disconnect_button) {
+	gtk_widget_set_sensitive(ui_layout.disconnect_button, 0);
+    }
+    if (ui_layout.forget_button) {
+	gtk_widget_set_sensitive(ui_layout.forget_button, 0);
+    }
+    if (ui_layout.verify_button) {
+	gtk_widget_set_sensitive(ui_layout.verify_button, 0);
+    }
     ui_layout.selected_fprint = NULL;
 }
 
@@ -143,7 +157,7 @@ static void clist_all_unselected(void)
 static void otrg_gtk_ui_update_keylist(void)
 {
     gchar *titles[5];
-    char hash[45];
+    char hash[OTRL_PRIVKEY_FPRINT_HUMAN_LEN];
     ConnContext * context;
     Fingerprint * fingerprint;
     int selected_row = -1;
@@ -161,26 +175,54 @@ static void otrg_gtk_ui_update_keylist(void)
 	int i;
 	PurplePlugin *p;
 	char *proto_name;
+
+	if (context->m_context != context) continue;
+
 	fingerprint = context->fingerprint_root.next;
 	/* If there's no fingerprint, don't add it to the known
 	 * fingerprints list */
 	while(fingerprint) {
+	    ConnContext *context_iter;
+	    TrustLevel best_level = TRUST_NOT_PRIVATE;
+	    int used = 0;
+
 	    titles[0] = context->username;
-	    if (context->msgstate == OTRL_MSGSTATE_ENCRYPTED &&
-		    context->active_fingerprint != fingerprint) {
-		titles[1] = _("Unused");
-	    } else {
+	    titles[1] = _("Unused");
+
+	    for (context_iter = context->m_context;
+		    context_iter && context_iter->m_context == context->m_context;
+		    context_iter = context_iter->next) {
+
+		TrustLevel this_level = TRUST_NOT_PRIVATE;
+
+		if (context_iter->active_fingerprint == fingerprint) {
+		    this_level = otrg_plugin_context_to_trust(context_iter);
+		    used = 1;
+
+		    if (this_level == TRUST_PRIVATE) {
+			best_level = TRUST_PRIVATE;
+		    } else if (this_level == TRUST_UNVERIFIED
+			    && best_level != TRUST_PRIVATE) {
+			best_level = TRUST_UNVERIFIED;
+		    } else if (this_level == TRUST_FINISHED
+			    && best_level == TRUST_NOT_PRIVATE) {
+			best_level = TRUST_FINISHED;
+		    }
+		}
+	    }
+
+	    if (used) {
 		titles[1] = (gchar *)
-		    _(trust_states[otrg_plugin_context_to_trust(context)]);
+		    _(trust_states[best_level]);
 	    }
 	    titles[2] = (fingerprint->trust && fingerprint->trust[0]) ?
-		_("Yes") : _("No");
+		    _("Yes") : _("No");
 	    otrl_privkey_hash_to_human(hash, fingerprint->fingerprint);
 	    titles[3] = hash;
 	    p = purple_find_prpl(context->protocol);
 	    proto_name = (p && p->info->name) ? p->info->name : _("Unknown");
 	    titles[4] = g_strdup_printf("%s (%s)", context->accountname,
-		proto_name);
+		    proto_name);
 	    i = gtk_clist_append(GTK_CLIST(keylist), titles);
 	    g_free(titles[4]);
 	    gtk_clist_set_row_data(GTK_CLIST(keylist), i, fingerprint);
@@ -207,9 +249,9 @@ static void generate(GtkWidget *widget, gpointer data)
 {
     PurpleAccount *account;
     account = pidgin_account_option_menu_get_selected(ui_layout.accountmenu);
-	
+
     if (account == NULL) return;
-	
+
     otrg_plugin_create_privkey(purple_account_get_username(account),
 	    purple_account_get_protocol_id(account));
 }
@@ -242,26 +284,34 @@ static void clist_selected(GtkWidget *widget, gint row, gint column,
     int verify_sensitive = 0;
     Fingerprint *f = gtk_clist_get_row_data(GTK_CLIST(ui_layout.keylist),
 	    row);
-    if (f && f->context->msgstate == OTRL_MSGSTATE_ENCRYPTED &&
-	    f->context->active_fingerprint == f) {
-	disconnect_sensitive = 1;
-    }
-    if (f && f->context->msgstate == OTRL_MSGSTATE_FINISHED) {
-	disconnect_sensitive = 1;
-    }
-    if (f && (f->context->msgstate != OTRL_MSGSTATE_ENCRYPTED ||
-	    f->context->active_fingerprint != f)) {
-	forget_sensitive = 1;
-    }
-    if (f && f->context->msgstate == OTRL_MSGSTATE_PLAINTEXT) {
-	connect_sensitive = 1;
-    }
-    if (f && f->context->msgstate == OTRL_MSGSTATE_FINISHED) {
-	connect_sensitive = 1;
-    }
+    ConnContext *context_iter;
+
     if (f) {
 	verify_sensitive = 1;
+	forget_sensitive = 1;
+
+	if (f->context && f->context->m_context) {
+	    for (context_iter = f->context;
+		    context_iter && context_iter->m_context ==
+		    f->context->m_context;
+		    context_iter = context_iter->next) {
+
+		if (context_iter->msgstate == OTRL_MSGSTATE_ENCRYPTED &&
+		    context_iter->active_fingerprint == f) {
+		    disconnect_sensitive = 1;
+		    forget_sensitive = 0;
+		}
+		else if (context_iter->msgstate == OTRL_MSGSTATE_FINISHED) {
+		    disconnect_sensitive = 1;
+		    connect_sensitive = 1;
+		}
+		else if (context_iter->msgstate == OTRL_MSGSTATE_PLAINTEXT) {
+		    connect_sensitive = 1;
+		}
+	    }
+	}
     }
+
     gtk_widget_set_sensitive(ui_layout.connect_button,
 	    connect_sensitive);
     gtk_widget_set_sensitive(ui_layout.disconnect_button,
@@ -277,25 +327,51 @@ static void clist_unselected(GtkWidget *widget, gint row, gint column,
     clist_all_unselected();
 }
 
+/* For a given fingerprint, find the master context that the fingerprint is
+ * pointing to, iterate through it and all its children.
+ * Of the contexts that are using this fingerprint, return a value that
+ * corresponds to the "best" trust level among these.
+ */
 static int fngsortval(Fingerprint *f)
 {
-    int is_active = (f->context->msgstate == OTRL_MSGSTATE_ENCRYPTED &&
-	    f->context->active_fingerprint == f);
-    TrustLevel level = otrg_plugin_context_to_trust(f->context);
+    int result = 200;
+    ConnContext *context_iter;
 
-    switch(level) {
-	case TRUST_PRIVATE:
-	    return is_active ? 0 : 100;
-	case TRUST_UNVERIFIED:
-	    return is_active ? 1 : 100;
-	case TRUST_FINISHED:
-	    return 2;
-	case TRUST_NOT_PRIVATE:
-	    return 3;
+    for (context_iter = f->context->m_context;
+	context_iter && context_iter->m_context == f->context->m_context;
+	context_iter = context_iter->next) {
+
+	int is_active = 0;
+	TrustLevel level;
+
+	if  (context_iter->msgstate == OTRL_MSGSTATE_ENCRYPTED &&
+	    context_iter->active_fingerprint == f) {
+	    is_active = 1;
+	}
+
+	level = otrg_plugin_context_to_trust(context_iter);
+
+	if (level == TRUST_PRIVATE) {
+	    if (is_active) {
+		result = 0;
+		break;
+	    } else {
+		result = result > 100 ? 100 : result;
+	    }
+	} else if (level == TRUST_UNVERIFIED) {
+	    if (is_active) {
+		result = 1;
+	    } else {
+		result = result > 100 ? 100 : result;
+	    }
+	} else if (level == TRUST_FINISHED) {
+	    result = result > 2 ? 2 : result;
+	} else if (level == TRUST_NOT_PRIVATE) {
+	    result = result > 3 ? 3 : result;
+	}
     }
 
-    /* Shouldn't get here, but anyway. */
-    return 200;
+    return result;
 }
 
 static gint statuscmp(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
@@ -344,20 +420,25 @@ static void disconnect_connection(GtkWidget *widget, gpointer data)
 {
     /* Forget whatever state we've got with this context */
     ConnContext *context;
+    ConnContext *context_iter;
 
     if (ui_layout.selected_fprint == NULL) return;
 
     context = ui_layout.selected_fprint->context;
     if (context == NULL) return;
-	
-    /* Don't do anything with fingerprints other than the active one
-     * if we're in the ENCRYPTED state */
-    if (context->msgstate == OTRL_MSGSTATE_ENCRYPTED &&
-	    context->active_fingerprint != ui_layout.selected_fprint) {
-	return;
+
+    for (context_iter = context->m_context;
+	    context_iter && context_iter->m_context == context->m_context;
+	    context_iter = context_iter->next) {
+
+	/* Don't do anything with fingerprints other than the active one
+	 * if we're in the ENCRYPTED state */
+	if (context_iter->msgstate == OTRL_MSGSTATE_ENCRYPTED &&
+	    context_iter->active_fingerprint == ui_layout.selected_fprint) {
+	    otrg_ui_disconnect_connection(context_iter);
+	}
     }
-	
-    otrg_ui_disconnect_connection(context);
+
 }
 
 static void forget_fingerprint(GtkWidget *widget, gpointer data)
@@ -382,7 +463,7 @@ static void otrsettings_clicked_cb(GtkButton *button,
 		GTK_TOGGLE_BUTTON(os->enablebox))) {
 	gtk_widget_set_sensitive(os->automaticbox, TRUE);
 	if (gtk_toggle_button_get_active(
-		    GTK_TOGGLE_BUTTON(os->automaticbox))) {
+		GTK_TOGGLE_BUTTON(os->automaticbox))) {
 	    gtk_widget_set_sensitive(os->onlyprivatebox, TRUE);
 	} else {
 	    gtk_widget_set_sensitive(os->onlyprivatebox, FALSE);
@@ -430,13 +511,13 @@ static void create_otrsettings_buttons(struct otrsettingsdata *os,
     gtk_box_pack_start(GTK_BOX(vbox), os->avoidloggingotrbox, FALSE, FALSE, 5);
 
     g_signal_connect(G_OBJECT(os->enablebox), "clicked",
-		     G_CALLBACK(otrsettings_clicked_cb), os);
+	    G_CALLBACK(otrsettings_clicked_cb), os);
     g_signal_connect(G_OBJECT(os->automaticbox), "clicked",
-		     G_CALLBACK(otrsettings_clicked_cb), os);
+	    G_CALLBACK(otrsettings_clicked_cb), os);
     g_signal_connect(G_OBJECT(os->onlyprivatebox), "clicked",
-		     G_CALLBACK(otrsettings_clicked_cb), os);
+	    G_CALLBACK(otrsettings_clicked_cb), os);
     g_signal_connect(G_OBJECT(os->avoidloggingotrbox), "clicked",
-		     G_CALLBACK(otrsettings_clicked_cb), os);
+	    G_CALLBACK(otrsettings_clicked_cb), os);
 }
 
 static void otroptions_clicked_cb(GtkButton *button,
@@ -450,17 +531,13 @@ static void otroptions_clicked_cb(GtkButton *button,
 static void create_otroptions_buttons(struct otroptionsdata *oo,
 	GtkWidget *vbox)
 {
-#ifdef OLD_OTR_BUTTON
-    oo->showotrbutton = gtk_check_button_new_with_label(_("Show OTR button"));
-#else
     oo->showotrbutton = gtk_check_button_new_with_label(
 	    _("Show OTR button in toolbar"));
-#endif
 
     gtk_box_pack_start(GTK_BOX(vbox), oo->showotrbutton, FALSE, FALSE, 0);
 
     g_signal_connect(G_OBJECT(oo->showotrbutton), "clicked",
-		     G_CALLBACK(otroptions_clicked_cb), oo);
+	    G_CALLBACK(otroptions_clicked_cb), oo);
 }
 
 /* Load the global OTR prefs */
@@ -477,8 +554,8 @@ static void otrg_gtk_ui_global_prefs_load(gboolean *enabledp,
 	*enabledp = TRUE;
 	*automaticp = TRUE;
 	*onlyprivatep = FALSE;
-	*avoidloggingotrp = FALSE;
-    }
+	*avoidloggingotrp = TRUE;
+  }
 }
 
 /* Save the global OTR prefs */
@@ -511,7 +588,7 @@ static void otrg_gtk_ui_buddy_prefs_load(PurpleBuddy *buddy,
 	*automaticp = purple_blist_node_get_bool(node, "OTR/automatic");
 	*onlyprivatep = purple_blist_node_get_bool(node, "OTR/onlyprivate");
 	*avoidloggingotrp =
-	    purple_blist_node_get_bool(node, "OTR/avoidloggingotr");
+		purple_blist_node_get_bool(node, "OTR/avoidloggingotr");
     }
 }
 
@@ -557,11 +634,7 @@ static void otrg_gtk_ui_global_options_load(gboolean *showotrbuttonp)
     if (purple_prefs_exists("/OTR/showotrbutton")) {
 	*showotrbuttonp = purple_prefs_get_bool("/OTR/showotrbutton");
     } else {
-#ifdef OLD_OTR_BUTTON
-	*showotrbuttonp = FALSE;
-#else
 	*showotrbuttonp = TRUE;
-#endif
     }
 }
 
@@ -685,13 +758,13 @@ static void make_settings_ui(GtkWidget *vbox)
     load_otrsettings(&(ui_layout.os));
 
     g_signal_connect(G_OBJECT(ui_layout.os.enablebox), "clicked",
-		     G_CALLBACK(otrsettings_save_cb), &(ui_layout.os));
+	    G_CALLBACK(otrsettings_save_cb), &(ui_layout.os));
     g_signal_connect(G_OBJECT(ui_layout.os.automaticbox), "clicked",
-		     G_CALLBACK(otrsettings_save_cb), &(ui_layout.os));
+	    G_CALLBACK(otrsettings_save_cb), &(ui_layout.os));
     g_signal_connect(G_OBJECT(ui_layout.os.onlyprivatebox), "clicked",
-		     G_CALLBACK(otrsettings_save_cb), &(ui_layout.os));
+	    G_CALLBACK(otrsettings_save_cb), &(ui_layout.os));
     g_signal_connect(G_OBJECT(ui_layout.os.avoidloggingotrbox), "clicked",
-		     G_CALLBACK(otrsettings_save_cb), &(ui_layout.os));
+	    G_CALLBACK(otrsettings_save_cb), &(ui_layout.os));
 }
 
 /* Make the options UI, and pack it into the vbox */
@@ -712,7 +785,7 @@ static void make_options_ui(GtkWidget *vbox)
     load_otroptions(&(ui_layout.oo));
 
     g_signal_connect(G_OBJECT(ui_layout.oo.showotrbutton), "clicked",
-		     G_CALLBACK(otroptions_save_cb), &(ui_layout.oo));
+	     G_CALLBACK(otroptions_save_cb), &(ui_layout.oo));
 }
 
 /* Create the fingerprint UI, and pack it into the vbox */
@@ -730,8 +803,8 @@ static void make_fingerprints_ui(GtkWidget *vbox)
     titles[4] = _("Account");
 
     ui_layout.scrollwin = gtk_scrolled_window_new(0, 0);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ui_layout.scrollwin), 
-            GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ui_layout.scrollwin),
+	    GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
 
     ui_layout.keylist = gtk_clist_new_with_titles(5, titles);
     gtk_clist_set_column_width(GTK_CLIST(ui_layout.keylist), 0, 90);
@@ -905,19 +978,19 @@ static void config_buddy_destroy_cb(GtkWidget *w, struct cbdata *data)
 static void config_buddy_clicked_cb(GtkButton *button, struct cbdata *data)
 {
     gboolean enabled = gtk_toggle_button_get_active(
-			     GTK_TOGGLE_BUTTON(data->os.enablebox));
-    
+	    GTK_TOGGLE_BUTTON(data->os.enablebox));
+
     /* Apply the changes */
     otrg_gtk_ui_buddy_prefs_save(data->buddy,
-	 gtk_toggle_button_get_active(
-	     GTK_TOGGLE_BUTTON(data->defaultbox)),
-	 enabled,
-	 gtk_toggle_button_get_active(
-	     GTK_TOGGLE_BUTTON(data->os.automaticbox)),
-	 gtk_toggle_button_get_active(
-	     GTK_TOGGLE_BUTTON(data->os.onlyprivatebox)),
-	 gtk_toggle_button_get_active(
-	     GTK_TOGGLE_BUTTON(data->os.avoidloggingotrbox)));
+	    gtk_toggle_button_get_active(
+	    GTK_TOGGLE_BUTTON(data->defaultbox)),
+		enabled,
+	    gtk_toggle_button_get_active(
+		GTK_TOGGLE_BUTTON(data->os.automaticbox)),
+	    gtk_toggle_button_get_active(
+		GTK_TOGGLE_BUTTON(data->os.onlyprivatebox)),
+	    gtk_toggle_button_get_active(
+		GTK_TOGGLE_BUTTON(data->os.avoidloggingotrbox)));
 
     otrg_dialog_resensitize_all();
 }
@@ -939,9 +1012,9 @@ static void otrg_gtk_ui_config_buddy(PurpleBuddy *buddy)
     if (!data) return;
 
     dialog = gtk_dialog_new_with_buttons(_("OTR Settings"),
-					 NULL, 0,
-					 GTK_STOCK_OK, GTK_RESPONSE_OK,
-					 NULL);
+	    NULL, 0,
+	    GTK_STOCK_OK, GTK_RESPONSE_OK,
+	    NULL);
     gtk_window_set_accept_focus(GTK_WINDOW(dialog), FALSE);
     gtk_window_set_role(GTK_WINDOW(dialog), "otr_settings");
 
@@ -985,25 +1058,25 @@ static void otrg_gtk_ui_config_buddy(PurpleBuddy *buddy)
     create_otrsettings_buttons(&(data->os), GTK_DIALOG(dialog)->vbox);
 
     g_signal_connect(G_OBJECT(data->defaultbox), "clicked",
-		     G_CALLBACK(default_clicked_cb), data);
+	    G_CALLBACK(default_clicked_cb), data);
     g_signal_connect(G_OBJECT(data->defaultbox), "clicked",
-		     G_CALLBACK(config_buddy_clicked_cb), data);
+	    G_CALLBACK(config_buddy_clicked_cb), data);
     g_signal_connect(G_OBJECT(data->os.enablebox), "clicked",
-		     G_CALLBACK(config_buddy_clicked_cb), data);
+	    G_CALLBACK(config_buddy_clicked_cb), data);
     g_signal_connect(G_OBJECT(data->os.automaticbox), "clicked",
-		     G_CALLBACK(config_buddy_clicked_cb), data);
+	    G_CALLBACK(config_buddy_clicked_cb), data);
     g_signal_connect(G_OBJECT(data->os.onlyprivatebox), "clicked",
-		     G_CALLBACK(config_buddy_clicked_cb), data);
+	    G_CALLBACK(config_buddy_clicked_cb), data);
     g_signal_connect(G_OBJECT(data->os.avoidloggingotrbox), "clicked",
-		     G_CALLBACK(config_buddy_clicked_cb), data);
+	    G_CALLBACK(config_buddy_clicked_cb), data);
 
     /* Set the inital states of the buttons */
     load_buddyprefs(data);
 
     g_signal_connect(G_OBJECT(dialog), "destroy",
-		     G_CALLBACK(config_buddy_destroy_cb), data);
+	    G_CALLBACK(config_buddy_destroy_cb), data);
     g_signal_connect(G_OBJECT(dialog), "response",
-		     G_CALLBACK(config_buddy_response_cb), data);
+	    G_CALLBACK(config_buddy_response_cb), data);
 
     gtk_widget_show_all(dialog);
 }
@@ -1015,12 +1088,12 @@ static void otrg_gtk_ui_get_prefs(OtrgUiPrefs *prefsp, PurpleAccount *account,
     PurpleBuddy *buddy;
     gboolean otrenabled, otrautomatic, otronlyprivate, otravoidloggingotr;
     gboolean buddyusedefault, buddyenabled, buddyautomatic, buddyonlyprivate,
-	     buddyavoidloggingotr;
+	    buddyavoidloggingotr;
 
     prefsp->policy = OTRL_POLICY_DEFAULT;
     prefsp->avoid_logging_otr = FALSE;
     prefsp->show_otr_button = FALSE;
-    
+
     /* Get the default policy */
     otrg_gtk_ui_global_prefs_load(&otrenabled, &otrautomatic, &otronlyprivate,
 	    &otravoidloggingotr);
